/**
 * mobios.db.PeptideTable 2006.07.24 
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log: 
 * 2006.07.24: Added, by Willard
 */
package mobios.db;


import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import mobios.dist.WeightMatrix;
import mobios.dist.SequenceFragmentMetric;
import mobios.dist.WHDGlobalSequenceFragmentMetric;
import mobios.type.Peptide;

public class PeptideTable extends SequenceTable {
    /**
     * 
     */
    private static final long                  serialVersionUID      = 5685357333502524772L;

    /**
     * 
     */
    public static final WeightMatrix           DEFAULT_WEIGHT_MATRIX = Peptide.mPAM250aExtendedWeightMatrix;

    /**
     * 
     */
    public static final SequenceFragmentMetric DEFAULT_METRIC        = new WHDGlobalSequenceFragmentMetric(
                                                                             DEFAULT_WEIGHT_MATRIX);

    /**
     * @param sourceData
     * @param maxSize
     * @param fragmentLength
     * @throws IOException
     */
    public PeptideTable(File sourceData, int maxSize, int fragmentLength) throws IOException {
        this(sourceData, maxSize, DEFAULT_METRIC, fragmentLength);
    }

    /**
     * @param sourceData
     * @param maxSize
     * @param metric
     * @param fragmentLength
     * @throws IOException
     */
    PeptideTable(File sourceData, int maxSize, SequenceFragmentMetric metric,
            int fragmentLength) throws IOException {
        super(sourceData, maxSize, metric, fragmentLength);
    }

    @Override
    protected void loadData(BufferedReader reader, int maxSize) {
        String ident = "";
        List<Peptide> seqs = new ArrayList<Peptide>();
        int counter = 0;
        int sequenceLengthCounter = 0;
        try {
            // read sequences from file
            StringBuffer currentSequence = new StringBuffer();
            String line = reader.readLine();
            if (line != null)
                line = line.trim();

            while (line != null && counter < maxSize && sequenceLengthCounter < maxSize) {
                if (line.length() >= 1) {
                    if (line.charAt(0) == '>') // beginning of a sequence
                    {
                        if (currentSequence.length() != 0) {
                            seqs.add(new Peptide(ident, currentSequence.toString()));
                            counter += currentSequence.length();
                            currentSequence.setLength(0);
                        }
                        ident = line;
                    } else
                    // begin of a new line of current sequence
                    {
                        currentSequence.append(line);
                        sequenceLengthCounter = currentSequence.length();
                    }
                }
                line = reader.readLine();
                if (line != null)
                    line = line.trim();
            }

            if (currentSequence.length() != 0)
                seqs.add(new Peptide(ident, currentSequence.toString()));
        } catch (java.io.IOException e) {
            throw new java.lang.IllegalStateException(
                    "Error occured when reading FASTA sequence file: " + reader
                            + " error message returned: " + e.getMessage());
        }
        sequences = new Peptide[seqs.size()];
        seqs.toArray(sequences);
    }
}
