/**
 * mobios.db.SequenceTable 2006.07.24 
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log: 
 * 2006.07.24: Added, by Willard
 */
package mobios.db;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import mobios.dist.SequenceFragmentMetric;
import mobios.type.Alphabet;
import mobios.type.Fragment;
import mobios.type.IndexObject;
import mobios.type.Sequence;
import mobios.type.Symbol;

/**
 * TODO
 * @author Willard
 *
 */
public abstract class SequenceTable extends Table {
    private static final long serialVersionUID = -1519446228237674948L;

    protected Alphabet                  alphabet;
    protected int                       fragmentLength;
    protected Sequence[]                sequences;
    protected int[]                     fragmentOffsets;


	/**TODO
     * @param fileName
     * @param maxSize
     * @param metric
     * @param fragmentLength
     * @throws IOException
     */
    SequenceTable(File sourceData, int maxSize, SequenceFragmentMetric metric,
            int fragmentLength) throws IOException {
        super(sourceData, maxSize, metric);
        if (fragmentLength <= 0)
            throw new IllegalArgumentException("fragment length must be greater than zero!");

        this.alphabet = metric.getWeightMatrix().getAlphabet();
        this.fragmentLength = fragmentLength;

        BufferedReader reader = new java.io.BufferedReader(new java.io.FileReader(sourceData));
        loadData(reader, maxSize);
        initFragmentList(maxSize);
    }

    /**TODO
     * @param reader
     * @param maxSize
     */
    protected abstract void loadData(BufferedReader reader, int maxSize);

    /**TODO
     * @param maxSize
     */
    private void initFragmentList(int maxSize) {
        int count = 0;
        // first figure out how long arrays are going to be;
        for (int i = 0; i < sequences.length; i++) {
            int numFragments = sequences[i].numFragments(fragmentLength);
            for (int j = 0; j < numFragments; j++) {
                if (count < maxSize) {
                    count++;
                }
            }
        }
        // init rowIDs list;
        ArrayList<Fragment> fragmentList = new ArrayList<Fragment>(count);
        originalRowIDs = new int[count];
        fragmentOffsets = new int[count];

        // reset count;
        count = 0;
        for (int i = 0; i < sequences.length; i++) {
            int numFragments = sequences[i].numFragments(fragmentLength);
            for (int j = 0; j < numFragments; j++) {
                if (count < maxSize) {
                    this.originalRowIDs[count] = i;
                    this.fragmentOffsets[count] = j;
                    Fragment frag = new Fragment(this, count);
                    fragmentList.add(frag);
                    count++;
                }
            }
        }

        fragmentList.trimToSize();
        data = fragmentList;
    }

    // TODO make better
    /* (non-Javadoc)
     * @see mobios.type.Table#compressData()
     */
    public void compressData() {
        // first sort the list according to the data points.
        Collections.sort(data);

        // then, make a list of the unique dataPoints.
        final int dataSize = data.size();
        ArrayList<IndexObject> compressedData = new ArrayList<IndexObject>(dataSize);
        int[] rowIDs2 = new int[dataSize];
        int[] dataOffset2 = new int[dataSize];

        IndexObject dataPoint1 = data.get(0);
        int tempSize = 1;

        IndexObject dataPoint2;
        for (int i = 1; i < dataSize; i++) {
            dataPoint2 = (IndexObject) data.get(i);
            if (dataPoint1.equals(dataPoint2)) {
                tempSize++;
            } else {
                if (tempSize > 1) {
                    for (int j = i - tempSize; j < i; j++) {
                        int rowID = data.get(j).getRowID();
                        rowIDs2[j] = originalRowIDs[rowID];
                        dataOffset2[j] = fragmentOffsets[rowID];
                    }
                    dataPoint1.setRowID(i - tempSize);
                    dataPoint1.setRowIDLength(tempSize);
                } else {
                    int rowID = data.get(i - 1).getRowID();
                    rowIDs2[i - 1] = originalRowIDs[rowID];
                    dataOffset2[i - 1] = fragmentOffsets[rowID];
                    dataPoint1.setRowID(i - 1);
                }
                compressedData.add(dataPoint1);
                dataPoint1 = dataPoint2;
                tempSize = 1;
            }
        }

        if (tempSize > 1) {
            for (int i = dataSize - tempSize; i < dataSize; i++) {
                int rowID = data.get(i).getRowID();
                rowIDs2[i] = originalRowIDs[rowID];
                dataOffset2[i] = fragmentOffsets[rowID];
            }
            dataPoint1.setRowID(dataSize - tempSize);
            dataPoint1.setRowIDLength(tempSize);
        } else {
            int rowID = data.get(dataSize - 1).getRowID();
            rowIDs2[dataSize - 1] = originalRowIDs[rowID];
            dataOffset2[dataSize - 1] = fragmentOffsets[rowID];
            dataPoint1.setRowID(dataSize - 1);
        }
        compressedData.add(dataPoint1);

        compressedData.trimToSize();
        System.out.println("original size: " + dataSize + " compressed data size: "
                + compressedData.size());
        data = compressedData;
        originalRowIDs = rowIDs2;
        fragmentOffsets = dataOffset2;
    }

    /**TODO
     * @return
     */
    public int getFragmentLength() {
        return fragmentLength;
    }

    /**TODO
     * @param rowID
     * @return
     */
    public int getFragmentOffset(int rowID) {
        return fragmentOffsets[rowID];
    }

    public Alphabet getAlphabet() {
    		return alphabet;
    }
    
	public Symbol getSymbol(int rowIDStart, int i) {
        return sequences[originalRowIDs[rowIDStart]].get(fragmentOffsets[rowIDStart] + i);
	}

	public String getSequenceID(int rowIDStart) {
		return sequences[originalRowIDs[rowIDStart]].getSequenceID();
	}

}
