/**
 * mobios.db.SpectraTable 2006.07.24 
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log: 
 * 2006.07.24: Added, by Willard
 */
package mobios.db;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import mobios.dist.MSMetric;
import mobios.dist.Metric;
import mobios.type.Spectra;

/**
 * TODO
 * @author Willard
 */
public class SpectraTable extends Table {
    /**
     * TODO
     */
    private static final long serialVersionUID = -6528032331962266589L;

    /**
     * TODO
     */
    public static final Metric DEFAULT_METRIC = new MSMetric();

    /**TODO
     * @param sourceData
     * @param maxSize
     * @throws FileNotFoundException
     */
    SpectraTable(File sourceData, int maxSize) throws IOException {
        this(sourceData, maxSize, DEFAULT_METRIC);
    }
    
    /**TODO
     * @param sourceData
     * @param maxSize
     * @param metric
     * @throws IOException
     */
    SpectraTable(File sourceData, int maxSize, Metric metric) throws IOException {
        super(sourceData, maxSize, metric);

        BufferedReader reader = new java.io.BufferedReader(new java.io.FileReader(sourceData));
        loadData(reader, maxSize);
    }

    /**TODO
     * @param reader
     * @param maxSize
     */
    private void loadData(BufferedReader reader, int maxSize) {
        String line;
        ArrayList<Spectra> spectra = null;
        System.out.println("Loading... ");
        try {
            // read sequences from file
            line = reader.readLine(); // read the first line
            if (line != null)
                line = line.trim();
            // get total rows from first line and allocate the ArrayList
            int numSpectra = java.lang.Integer.parseInt(line);
            spectra = new ArrayList<Spectra>(numSpectra);
            originalRowIDs = new int[numSpectra];

            // read line byte line
            line = reader.readLine();
            if (line != null)
                line = line.trim();

            int count = 0;
            while (line != null && count < maxSize) {
                if (count % 1000 == 0) {
                    System.out.print(count + ".. ");
                }
                String[] row = line.split(" ", 2);
                originalRowIDs[count] = new Integer(row[0]).intValue();
                spectra.add(new Spectra(this, count, row[1]));

                line = reader.readLine();
                if (line != null)
                    line = line.trim();
                count++;
            }
        } catch (java.io.IOException e) {
            e.printStackTrace();
            throw new java.lang.IllegalStateException("Error occured when reading ms file: "
                    + reader + " error message returned: " + e.getMessage());
        } catch (NumberFormatException e) {
            e.printStackTrace();
            /* Ignore strings with invalid characters. */
        }
        spectra.trimToSize();
        data = spectra;
    }

}
