/**
 * mobios.db.TandemSpectraTable 2006.07.31
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 *
 */
package mobios.db;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;

import mobios.dist.MSMSMetric;
import mobios.dist.Metric;
import mobios.type.TandemSpectra;

/**
 * TODO comment this entire file
 * 
 * 
 * @author willard
 *
 */
public class TandemSpectraTable extends Table
{
    /**
     * 
     */
    private static final long serialVersionUID = -6784466397812636659L;

    /**
     * 
     */
    public static final Metric DEFAULT_METRIC = new MSMSMetric();
    
    /**
     * @param sourceData
     * @param maxDataSize
     * @throws FileNotFoundException
     */
    public TandemSpectraTable(File sourceData, int maxDataSize) throws FileNotFoundException {
        this(sourceData, maxDataSize, DEFAULT_METRIC);
    }
    
    /**
     * @param sourceData
     * @param size
     * @param metric
     * @throws FileNotFoundException
     */
    TandemSpectraTable(File sourceData, int size, Metric metric) throws FileNotFoundException {
        super(sourceData, size, metric);
        
        BufferedReader reader = new java.io.BufferedReader(new java.io.FileReader(sourceData));
        loadData(reader, size);   
    }

    /**
     * @param reader
     * @param size
     */
    private void loadData(BufferedReader reader, int size) {
        
        String line;
        ArrayList<TandemSpectra> spectraWPM = null;
        ArrayList<Integer> originalRowIDsArrayList = null;
        System.out.println("Loading... ");
        try
        {
            // read sequences from file
            line = reader.readLine(); // read the first line
            if (line != null)
                line = line.trim();
            // get total rows from first line and allocate the ArrayList
            int numSpectra = java.lang.Integer.parseInt(line);
            spectraWPM = new ArrayList<TandemSpectra>(numSpectra);
            originalRowIDsArrayList = new ArrayList<Integer>(numSpectra);
            
            // read line byte line
            line = reader.readLine();
            if (line != null)
                line = line.trim();

            int count = 0;
            while (line != null && count < size)
            {
                if (count % 1000 == 0)
                {
                    System.out.print(count+".. ");
                }
                String[] row = line.split(" ", 3);
                originalRowIDsArrayList.add(count,new Integer(row[0]).intValue());
                spectraWPM.add(new TandemSpectra(this, count, new Double(row[1]).doubleValue(), row[2]));

                line = reader.readLine();
                if (line != null)
                    line = line.trim();
                count++;
            }
        }
        catch (java.io.IOException e)
        {
            e.printStackTrace();
            throw new java.lang.IllegalStateException("Error occured when reading msms file: " + reader
                    + " error message returned: " + e.getMessage());
        }
        catch (NumberFormatException e)
        {
            e.printStackTrace();
            /* Ignore strings with invalid characters. */
        }

        // debug
        // System.out.println(seqs);
        spectraWPM.trimToSize();
        data = spectraWPM;

        originalRowIDs = new int [originalRowIDsArrayList.size()];
        for (int i =0, e = originalRowIDsArrayList.size(); i < e; i++) {
            originalRowIDs[i] = originalRowIDsArrayList.get(i);
        }
    }

}
