/**
 * mobios.dist.CountedMetric 2006.05.24
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2006.05.24: Modified from the original jdb package, by Willard
 */

package mobios.dist;

import mobios.type.IndexObject;

/**
 * Wrapper around a base {@link Metric} that counts the invocations of
 * {@link Metric#getDistance(IndexObject,IndexObject)}. Additional methods to get and clear the
 * counter are provided.
 * 
 * @author Jack, Rui Mao, Willard
 * @version 2004.03.05
 */
public class CountedMetric implements Metric {
    /**
     * 
     */
    private static final long serialVersionUID = 5436226220280070858L;

    /**
     * 
     */
    final private Metric      baseMetric;
    
    /**
     *
     */
    private int               counter;

    /**
     * Creates a <code>CountedMetric</code> with a given base {@link Metric}, setting the
     * internal counter to zero.
     */
    public CountedMetric(Metric baseMetric) {
        if (baseMetric == null)
            throw new NullPointerException("object baseMetric cannot be null");
        this.baseMetric = baseMetric;
        this.counter = 0;
    }

    /**
     * Returns the value of {@link Metric#getDistance(IndexObject,IndexObject)} for the base
     * {@link Metric} and increments the internal counter.
     */
    final public double getDistance(IndexObject one, IndexObject two) {
        ++counter;
        return baseMetric.getDistance(one, two);
    }

    /**
     * Returns the current value of the internal counter.
     */
    final public int getCounter() {
        return counter;
    }

    /**
     * Sets the internal counter to zero.
     */
    final public void clear() {
        counter = 0;
    }

}
