/**
 * mobios.dist.MSMetric 2004.03.17
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2004.05.31; Modified for performance, replaced ArrayLists with arrays, by Willard
 * 2004.08.02; Modified to implement MSMSConstants, by Smriti Ramakrishnan
 */

package mobios.dist;

import mobios.dist.Metric;
import mobios.type.IndexObject;
import mobios.type.Spectra;
import static mobios.dist.MSMSConstants.MS_TOLERANCE;

/**
 * MSDataMetric implements the {@link Metric} interface. 
 * 
 * @author Smriti Ramakrishnan
 * @version 2004.03.25
 **/

public class MSMetric implements Metric {
    static final long serialVersionUID = 179878801379011989L; 
    
	public MSMetric(int min, int max, double step, double tol) {
		//super(min, max, step);    
		this.min = min;
		this.max = max;
		this.step = step;
		this.tol = tol;
	}

	/** default constructor - is a temporary solution providing default values for
	 *	min, max, step, tolerance
	 *
	 */
	public MSMetric() {
		this.min = 0;
		this.max = 0;
		this.step = 0;
		this.tol = MS_TOLERANCE;
	}

	/**
	 * @param v1 the {@link Object} over which the keys are defined.
	 * @param v2 the other {@link Object} over which the keys are defined.
	 */
	public double getDistance(IndexObject v1, IndexObject v2) {
		return getDistance((Spectra)v1, (Spectra)v2);
	}

	/**
	 * @param v1 the {@link MSKeyObject} over which the keys are defined.
	 * @param v2 the other {@link MSKeyObject} over which the keys are defined.
	 */
	public double getDistance(Spectra v1, Spectra v2) {
		// now compute cosine distance
		double cos = getCosine(v1, v2);

		// from BinaryVectorMetric:getDistance()
		//if ( Math.abs(cos) > 1) {; Smriti

		if (Math.abs(Math.abs(cos) - 1) < COS_THRESHOLD) { // precision
			if (cos > 0)
				cos = 1; // very similar
			else
				cos = -1; // very unsimilar
		} else if (Math.abs(cos) > 1) {
			System.out.println("COS_THRESHOLD = " + COS_THRESHOLD + ", cos = " + cos);
			System.out.println("got cosine > 1, cosine=" + cos + ", :" + v1.toString() + ", v2:" + v2.toString() + "Quitting.");
			System.exit(0);
		}
		//}
		/*
		System.out.println("MSTOL = " + MS_TOLERANCE);
		System.out.println("MSDKO-1 : " + v1);
		System.out.println("MSDKO-2 : " + v2);
		System.out.println("Dist = "  + Math.acos(cos) + ",Cosine = " + cos);
		System.out.println();
		System.out.println();
		*/
		return Math.acos(cos);


		// smriti - 30 sep
		//return  (1 - cos);
		/*
		//extended jaccard
		
		double[] one = v1.getData();
		double[] two = v2.getData();
		
		double spc = getInnerProduct(one, two);
		double dist = 1 - (spc / (getMagnitude(one) + getMagnitude(two) - spc));
		return dist;*/
		//getCosine(,v2));//weijia
	}

	private double getCosine(Spectra one, Spectra two) {
		double[] v1 = one.getData();
		double[] v2 = two.getData();
		
		return getInnerProduct(v1, v2) / (getMagnitude(v1) * getMagnitude(v2));
		
		// smriti - 30sep
		/*
		double numer = getInnerProduct(v1, v2) ;
		double denom = getMagnitude(v1) * getMagnitude(v2);
		
		double denom = getMagnitude(v1) * getMagnitude(v1);
		double d2 = getMagnitude(v2) * getMagnitude(v2);
		denom = (denom + d2) / 2;
		
		return numer/denom;
		*/
	}

	/** This is the method that replaces calls to super.getInnerProduct()
	  * Compute FuzzyInnerProduct withing a certain tolerance.
	  */
	private int getInnerProduct(double[] v1, double[] v2) {
		int dist = 0;
		int i = 0, j = 0;
		double val1;
		double val2;

		//System.out.println("TOL = " + tol);
		while (i < v1.length && j < v2.length) {
			val1 = v1[i];
			val2 = v2[j];
			//new version -- gets rid of Math.abs()
			if (val1 <= val2) {
				if (val2 - val1 <= tol) {
					dist++;
					i++;
					j++;
				} else {
					i++;
				}
			} else {
				if (val1 - val2 <= tol) {
					dist++;
					i++;
					j++;
				} else {
					j++;
				}
			}

			/*old code
			if (Math.abs(val1 - val2) <= tol) {
				dist++;
				i++;
				j++;
			} else {
				if (val1 < val2)
					i++;
				else
					j++;
			}*/
			//debug
			//System.out.println("i = " + i + ", j = " + j);		
		}
		
		return dist;
	}

	/**
	* For experiment purpose only
	* Method to compute inner product, cosine value and distance of v2
	* {@link VectorData} objects by treating them as binary vectors in a
	* multidimensional space.The Value will be stored in a {@link double[]}
	* with the correspoding order.
	* @param  - {@link VectorData}
	* @param v2 - {@link vectorData}
	* @return {@link double []}
	**/
	/*public double[] getAll(DataVector one, DataVector two) {
		double[] v1 = one.getData();
		double[] v2 = two.getData();
		double[] result = new double[3];
		result[0] = getInnerProduct(v1, v2);
		result[1] = result[0] / (getMagnitude(v1) * getMagnitude(v2));
		result[2] = Math.(result[1]);
		
		return result;
	}*/

	/**
	* Computes the magnitude of the {@link VectorData} object by
	* treating it as a binary vector
	* @param {@link VectorData}
	**/
	private double getMagnitude(double[] d) {
		return Math.sqrt(d.length);
		//return d.length;
	}

	/**
	 *Convert information of the instance into a {@link String} that
	*contains the number of dimension and meaning for each dimension
	**/
	/* public String toString()
	{
			String result="incomplete implementation";
		return result;
	 }*/

	//-- Changing this for mSQL serializability reqmt - dont need custom functions here anyway
	//legacy
	int min, max; // min, max are mass ranges
	double step; // Same as Tolerance ?
	//end-legacy

	double tol;
	private final double COS_THRESHOLD = 0.00005;
}
