/**
 * mobios.dist.Metric 2006.05.24
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2006.05.04: Copied from jdb package, by Rui Mao
 * 2006.05.24: Changed to serializable interface, by Willard
 */

package mobios.dist;

import java.io.Serializable;

import mobios.type.IndexObject;

/**
 * <code>Metric</code> is a distance oracle. It specifies a binary function to compute distance
 * between two data points in metric space. The distance function should capture the metric
 * properties, i.e. non-negativity, symmetry and the triangle inequality.
 * 
 * @author Jack, Rui Mao, Willard
 * @version 2005.10.31
 */
public interface Metric extends Serializable {

    /**
     * Computes the distance between two objects.
     * 
     * @param one
     *        the first {@link IndexObject} to compute distance on
     * @param two
     *        the second {@link IndexObject} to compute distance on
     * @return the distance between the two objects
     */
    public double getDistance(IndexObject one, IndexObject two);
}
