/**
 * mobios.dist.SequenceFragmentMetric 2006.05.24
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2006.05.24: Modified from the original jdb package, by Willard
 */
package mobios.dist;

import mobios.type.Fragment;
import mobios.type.IndexObject;

/**
 * Computes the distance between two {@link Fragment}s.
 * 
 * @author Jack, Rui Mao, Willard
 * @version 2006.05.31
 */
public abstract class SequenceFragmentMetric implements Metric {
    /**
     * the {@link WeightMatrix} that this SequenceFragmentMetric is defined over.
     */
    protected WeightMatrix weightMatrix;

    /**
     * Creates a new SequenceFragmentMetric over the given {@link WeightMatrix}.
     * 
     * @param weightMatrix
     */
    public SequenceFragmentMetric(WeightMatrix weightMatrix) {
        this.weightMatrix = weightMatrix;
    }

    /**
     * @return the {@link WeightMatrix} used when computing the distance between two fragments using
     *         this metric.
     */
    public WeightMatrix getWeightMatrix() {
        return weightMatrix;
    }

    /*
     * (non-Javadoc)
     * 
     * @see mobios.dist.Metric#getDistance(mobios.type.IndexObject, mobios.type.IndexObject)
     */
    public double getDistance(IndexObject one, IndexObject two) {
        return getDistance((Fragment) one, (Fragment) two);
    }

    /**
     * Computes the distance between two {@link Fragment}s
     * 
     * @param one
     *        the first {@link Fragment} to compute distance on
     * @param two
     *        the second {@link Fragment} to compute distance on
     * @return the distance between the two {@link Fragment}s
     */
    public abstract double getDistance(Fragment one, Fragment two);
}
