/**
 * mobios.dist.SymmetricSubstitutionWeightMatrix 2006.05.24
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2006.05.24: Modified from the original jdb package, by Willard; original class: WeightedAlphabetMetric 
 */

package mobios.dist;

import mobios.type.Alphabet;
import mobios.type.Symbol;

/**
 * Implements a {@link WeightMatrix} by providing a symmetric substitution matrix of double
 * distances for a given {@link Alphabet}. Thus, this class defines a weighted edit distance on an
 * {@link Alphabet}.
 * 
 * @author Rui Mao, Willard
 * @version 2004.03.02
 */
public class SymmetricSubstitutionWeightMatrix implements WeightMatrix {
    /**
     * 
     */
    private static final long serialVersionUID = 6295557519158168768L;

    private Alphabet          alphabet;
    private int               alphabetSize;
    private double[][]        distances;

    /**
     * Constructor.
     * 
     * @param alphabet
     *        the {@link Alphabet} that this SymmetricSubstitionWeightMatrix is defined over.
     * @param distances
     *        a two-dimensional array of double distances between the {@link Symbol} objects of the
     *        <code>alphabet</code>.
     */
    public SymmetricSubstitutionWeightMatrix(Alphabet alphabet, double[][] distances) {
        this.alphabetSize = alphabet.distinctSize();
        int minLength = alphabetSize;
        if (minLength > distances.length)
            throw new IndexOutOfBoundsException("array \"distances\" is length " + distances.length
                    + "; too small for \"alphabet\" of size " + alphabetSize);
        for (int i = 0; i < distances.length; i++) {
            if (minLength > distances[i].length) {
                throw new IndexOutOfBoundsException("array \"distances\" is length "
                        + distances.length + "; too small for \"alphabet\" of size " + alphabetSize);
            }
        }

        this.alphabet = alphabet;
        this.distances = distances;
    }

    /*
     * (non-Javadoc)
     * 
     * @see mobios.dist.WeightMatrix#getDistance(mobios.type.Symbol, mobios.type.Symbol)
     */
    public double getDistance(Symbol one, Symbol two) {
        return distances[one.byteValue()][two.byteValue()];
    }

    /*
     * (non-Javadoc)
     * 
     * @see mobios.dist.WeightMatrix#getAlphabet()
     */
    public Alphabet getAlphabet() {
        return alphabet;
    }
}
