/**
 * mobios.dist.WHDGlobalSequenceFragmentMetric 2006.05.24
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2006.05.24: Modified from the original mobios package, by Willard
 */

package mobios.dist;

import mobios.type.Fragment;

/**
 * Computes global alignment on {@link Fragment}s with Weighted Hamming Distance. The fragments
 * must be the same length.
 * 
 * @author Weijia Xu, Rui Mao, Willard
 * @version 2004.03.02
 */

public class WHDGlobalSequenceFragmentMetric extends SequenceFragmentMetric {
    /**
     * 
     */
    private static final long serialVersionUID = 7847936320149830952L;

    /**
     * Constructor. Defines a new WHDGlobalSequenceFragmentMetric over the given {@link WeightMatrix}
     * 
     * @param weightMatrix
     */
    public WHDGlobalSequenceFragmentMetric(WeightMatrix weightMatrix) {
        super(weightMatrix);
    }

    
    /* (non-Javadoc)
     * @see mobios.dist.SequenceFragmentMetric#getDistance(mobios.type.Fragment, mobios.type.Fragment)
     */
    public double getDistance(Fragment one, Fragment two) {
        int firstSize;
        if ((firstSize = one.size()) != two.size()) {
            throw new IllegalArgumentException("Fragments must have the same length." +
                    " Fragment one size: "+one.size()+" Fragment two size: "+two.size());
        }
        double distance = 0.0;
        for (int i = 0; i < firstSize; i++) {
            distance += weightMatrix.getDistance(one.get(i), two.get(i));
        }
        return distance;
    }

    /* (non-Javadoc)
     * @see mobios.dist.SequenceFragmentMetric#getWeightMatrix()
     */
    public WeightMatrix getWeightMatrix() {
        return weightMatrix;
    }
}
