/**
 * mobios.dist.WeightMatrix 2006.05.24
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Change Log:
 * 2006.05.24: Modified from the original mobios package, by Willard; original class was called:AlphabetMetric
 */

package mobios.dist;

import java.io.Serializable;

import mobios.type.Alphabet;
import mobios.type.Fragment;
import mobios.type.Symbol;
import mobios.type.Sequence; // for javadoc

/**
 * A <code>WeightMatrix</code> is the substitution matrix of an {@link Alphabet}of {@link Symbol}
 * objects. It defines the distance between each pair of <code>Symbol</code>s of the
 * <code>Alphabet</code>.  Using a <code>WeightMatrix</code>, a weighted edit distance metric can be defined
 * on{@link Sequence} or {@link Fragment} objects.
 *
 * @author Jack, Rui Mao, Weijia Xu, Willard
 * @version 2004.03.02
 */
public interface WeightMatrix extends Serializable {
    /**
     * @return the {@link Alphabet} over which this matrix is defined.
     */
    public Alphabet getAlphabet();

    /**
     * @return the distance between two {@link Symbol} objects in the {@link Alphabet} over which
     *         this matrix is defined.
     */
    public double getDistance(Symbol one, Symbol two);

}
