/**
 * mobios.index.Index 2006.05.09
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2006.05.09: Created, by Rui Mao, Willard
 */

package mobios.index;

import java.io.IOException;
import java.io.Serializable;
import java.util.List;

import mobios.type.IndexObject;
import mobios.util.ObjectIOManager;
import mobios.dist.Metric;

/**
 * The primary interface for distance-based index. Through Index, the user can build a database
 * index, or read a pre-built index from a file, and then do the search.
 * 
 * @author Rui Mao, Willard Saint Willard
 * @version 2006.05.09
 */
public interface Index extends Serializable {

    /**
     * @return the metric used to build the index.
     */
    public Metric getMetric();

    /**
     * @return the total number of data objects contained in the index.
     */
    public int size();

    /**
     * When a lot of queries are to be answered, for the sake of performance, it is a good idea to
     * have the top levels of the index tree reside in memory. This method loads the top levels into
     * memory.
     * 
     * @param level
     *        number of levels, from the root, to be pre-loaded into memory.
     */
    public void preLoad(int level);

    /**
     * Executes a range query. For a range query (q,r), the results should be all database points
     * satisfying d(q,x)<=r.
     * 
     * @param query
     *        the {@link Query} object
     * @return a {@link Cursor} over the results of the query
     */
    public Cursor search(Query query);

    /**
     * Returns all the data points in the index
     */
    List<IndexObject> getAllPoints();

    /**
     * Closes anything used internally that needs to be closed. It is a good idea to close it when
     * it is no longer in use.
     * 
     * @throws IOException
     */
    public void close() throws IOException;

    /**
     * Deletes the index from the file, and also release it from memory. This method should only be
     * called when the index will never be used again!
     * 
     * @throws IOException
     */
    public void destroy() throws IOException;

    /**
     * TODO
     * @return
     */
	public ObjectIOManager getOIOM();

	/**
	 * TODO
	 * @return
	 */
	public long getRootAddress();
}