/**
 * mobios.index.LeafNode 2006.05.12
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2006.05.12: created, by Rui Mao, Willard
 */

package mobios.index;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import mobios.type.IndexObject;

/**
 * The base class for all leaf nodes.
 * 
 * @author Rui Mao, Willard
 * @version 2006.05.12
 */

abstract class LeafNode extends IndexNode {
    /**
     * 
     */
    protected IndexObject[] children;

    /**
     * Default constructor provided for implementing {@link Externalizable}. Normally will use
     * {@link LeafNode#LeafNode(IndexObject[], IndexObject[], int)}
     */
    public LeafNode() {
        super();
    }

    /**
     * Constructor for a leaf node. 
     * TODO
     * 
     * @param pivots
     * @param pivotRowIDs
     * @param children
     * @param childRowIDs
     * @param size
     */
    protected LeafNode(IndexObject[] pivots, IndexObject[] children, int size) {
        super(pivots, size);
        if (children == null)
            throw new IllegalArgumentException("LeafNode children cannot be null");
        this.children = children;
    }

    /**
     * @return the number of children.
     */
    int numChildren() {
        return children.length;
    }

    /**
     * Return a distinct key value of children.
     * 
     * @param dataIndex
     *        index of the distinct key value to be return
     * @return the key value
     */
    IndexObject getChild(int dataIndex) {
        return children[dataIndex];
    }

    /*
     * (non-Javadoc)
     * 
     * @see mobios.index.IndexNode#writeExternal(java.io.ObjectOutput)
     */
    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeInt(children.length);
        for (int i = 0; i < children.length; i++) {
            out.writeObject(children[i]);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see mobios.index.IndexNode#readExternal(java.io.ObjectInput)
     */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);
        children = new IndexObject[in.readInt()];
        for (int i = 0; i < children.length; i++) {
            children[i] = (IndexObject) in.readObject();
        }
    }
}