/**
 * mobios.index.RangeQuery 2006.05.09
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2006.05.09: Modified from the original mobios package, ProximityPredicate.java, by Rui Mao
 */

package mobios.index;

import mobios.type.IndexObject;

/**
 * A range query. Given (q,r), the results should be all database points x such that d(x,q) <= r
 * @author Rui Mao, Willard
 * @version 2006.05.09
 */
public class RangeQuery implements Query 
{
    /**
     * Initializes the range query object
     *
     * @param center the {@link Object} that serves as the query object
     * @param radius the search radius of the range query. 
     **/
    public RangeQuery (IndexObject center, double radius)
    {
        this(center, radius,0);
    }
    
    public RangeQuery (IndexObject center, double radius, int listSize) 
    {
        if (radius < 0.0)
            throw new IllegalArgumentException ("radius < 0: " + radius) ;
        
        if (listSize < 0)
            throw new IllegalArgumentException ("max distance list size < 0: " + listSize) ;

        this.radius = radius ;
        this.center = center ;
        this.listSize = listSize;
    }
    
    /**
     * Return a reference to the query object
     * @return a reference to the query object
     */
    public IndexObject getQueryObject()
    {
        return center;
    }


    /**
     * Returns the search radius of the proximity query.
     */
    final public double getRadius () 
    {
        return radius ;
    }
    
    public int getMaxDistanceListSize()
    {
        return listSize;
    }

    private double radius ;
    private IndexObject center;
    private int listSize;

}
