/**
 * mobios.type.Alphabet 2006.05.24
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2006.05.24: Created, by Willard
 */
package mobios.type;

import java.io.Serializable;

/**
 * An alphabet represents a collection of {@link Symbol}s for a specific {@link Sequence} type.
 * 
 * @author Willard
 */
public class Alphabet implements Serializable {
    /**
     * 
     */
    private static final long serialVersionUID = -3200343173661145001L;

    private Symbol[]                  alphabet;
    
    int                       distinctSize;

    /**
     * Main constructor.
     *
     * @param alphabet An array of {@link Symbol} objects that this alphabet is defined over.
     * @param distinctSize the number of distinct byteValues that the symbols in the alphabet link to.
     */
    public Alphabet(Symbol[] alphabet, int distinctSize) {
        this.alphabet = alphabet;
        this.distinctSize = distinctSize;
    }

    /**
     * 
     * @return the size of the alphabet. May be greater than or equal to the distinctSize, but not smaller.
     */
    public int size() {
        return alphabet.length;
    }

    /**
     * @param index the index of the desired {@link Symbol} in the Symbol array.
     * @return the desired {@link Symbol}.
     */
    public Symbol get(int index) {
        return alphabet[index];
    }

    /**
      * Two {@link Symbol}s may map to the same byteValue; this is used
      * to determine the number of distinct byteValues that an Alphabet has.
      * See {@link Symbol} for more info.
      * 
     * @return
     */
    public int distinctSize() {
        return distinctSize;
    }
}
