/**
 * mobios.type.DoubleObjectPair 2003.07.19
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2003.07.19: Modified from the original mobios package, by Rui Mao
 * 2004.10.31: add toString(), by Rui Mao
 */
package mobios.type;

import java.util.Comparator;

/**
 * Wraps a <code>double</code> and an {@link IndexObject}.
 * It also contains two {@link Comparator}s to compare the double or the {@link IndexObject}.
 * @author Rui Mao, Willard
 * @version 2006.07.31
 */
public class DoubleIndexObjectPair
{
    /**
     * 
     */
    private double _double;
    /**
     * 
     */
    private IndexObject object;

    
    /**
     * @param _double
     * @param _iobject
     */
    public DoubleIndexObjectPair(double _double, IndexObject _iobject)
    {
		this._double = _double;
		this.object = _iobject;
    }

    /**
     * 
     */
    public DoubleIndexObjectPair()
    {
		this._double = 0;
		this.object = null;
    }

    /**
     * @return
     */
    public double getDouble()
    {
		return _double;
    }

    /**
     * @return
     */
    public IndexObject getObject()
    {
		return object;
    }

    /**
     * @param _double
     */
    public void setDouble( double _double)
    {
		this._double = _double;
    }

    /**
     * @param _iobject
     */
    public void setObject(IndexObject _iobject)
    {
		this.object = _iobject;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
    	return "double =" + _double + ", object= " + object; 
    }

    /**
     * 
     */
    public static final Comparator<DoubleIndexObjectPair> DoubleComparator = new Comparator<DoubleIndexObjectPair>()
    {
		public int compare(DoubleIndexObjectPair first, DoubleIndexObjectPair second)
		{
			final double firstDouble  = first.getDouble();
			final double secondDouble = second.getDouble();
			return firstDouble < secondDouble ? -1 : firstDouble > secondDouble ? 1 :0;
		}
	};

    /**
     * 
     */
    public static final Comparator<DoubleIndexObjectPair> ObjectComparator = new Comparator<DoubleIndexObjectPair>()
    {
        public int compare(DoubleIndexObjectPair first, DoubleIndexObjectPair second)
		{
			return first.getObject().compareTo(second.getObject());
		}
	};
}
