/**
 * mobios.type.Fragment 2006.05.24
 *
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2006.05.24: Created, by Willard
 */
package mobios.type;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import mobios.db.SequenceTable;
import mobios.db.TableManager;

/**
 * TODO
 * 
 * @author Willard
 * @version 2006.07.31
 */
public class Fragment extends IndexObject
{
    /**
     * TODO
     */
    private static final long serialVersionUID = -7087849259510041868L;

    /**
     * TODO
     */
    private SequenceTable     sTable;

    /**
     * TODO
     */
    public Fragment()
    {
    }

    /**
     * TODO
     * 
     * @param table
     * @param rowID
     */
    public Fragment(SequenceTable table, int rowID)
    {
        super(rowID);
        this.sTable = table;
    }

    /**
     * TODO
     * 
     * @param i
     * @return
     */
    public Symbol get(int i)
    {
    		return sTable.getSymbol(rowIDStart, i);
    		}
    
    /* (non-Javadoc)
     * @see mobios.type.IndexObject#size()
     */
    public int size()
    {
        return sTable.getFragmentLength();
    }

    /* (non-Javadoc)
     * @see mobios.type.IndexObject#expand()
     */
    public IndexObject[] expand()
    {
        IndexObject[] dbO = new IndexObject[rowIDLength];
        for (int i = 0; i < rowIDLength; i++)
        {
            dbO[i] = new Fragment(sTable, rowIDStart + i);
        }
        return dbO;
    }

    /* (non-Javadoc)
     * @see mobios.type.IndexObject#compareTo(mobios.type.IndexObject)
     */
    public int compareTo(IndexObject oThat)
    {
        if (!(oThat instanceof Fragment))
            throw new Error("not compatible");
        Fragment that = (Fragment) oThat;
        if (this == that)
            return 0;

        if (this.size() < that.size())
            return -1;
        else if (this.size() > that.size())
            return 1;
        else
        {
        		int fragLength = sTable.getFragmentLength();
            for (int i = 0; i < fragLength; i++)
            {
                byte byte1 = get(i).byteValue();
                byte byte2 = that.get(i).byteValue();
                if (byte1 < byte2)
                    return -1;
                else if (byte1 > byte2)
                    return 1;
            }
            return 0;
        }

    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public final boolean equals(Object object)
    {
        if (object == null)
            return false;

        if (!(object instanceof Fragment))
            return false;

        Fragment that = (Fragment) object;

        if (this.size() != that.size())
            return false;

        if (!this.sTable.getAlphabet().equals(that.sTable.getAlphabet()))
            return false;

        for (int i = 0; i < size(); ++i)
            if (this.get(i) != that.get(i))
                return false;
        return true;
    }
    
    //  taken from Joshua Bloch's Effective Java
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        int result = super.hashCode();
        result = 37*result+size();
        result = 37*result+sTable.getAlphabet().hashCode();
        for (int i = 0; i < size(); ++i) {
            result=37*result+get(i).hashCode();
        }
        return result;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        StringBuffer sourceSequence = new StringBuffer(rowIDLength);
        for (int i = 0; i < rowIDLength; i++)
        {
            sourceSequence.append(sTable.getSequenceID(rowIDStart)).append(", offset: ").append(sTable.getFragmentOffset(rowIDStart + i)).append(" ");
        }
        int fragLength = sTable.getFragmentLength();
        StringBuffer fragment = new StringBuffer(fragLength);
        for (int i = 0; i < fragLength; i++)
        {
            fragment.append(get(i));
        }
        return "fragment: " + fragment.toString() + " source: " + sourceSequence.toString();
    }

    /* (non-Javadoc)
     * @see mobios.type.IndexObject#readExternal(java.io.ObjectInput)
     */
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
    {
        super.readExternal(in);
        sTable = (SequenceTable) TableManager.getTableManager().getTable(in.readInt());
    }

    /* (non-Javadoc)
     * @see mobios.type.IndexObject#writeExternal(java.io.ObjectOutput)
     */
    public void writeExternal(ObjectOutput out) throws IOException
    {
        super.writeExternal(out);
        out.writeInt(sTable.getTableLocation());
    }
}
