/**
 * mobios.type.IndexObject 2006.07.24 
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log: 
 * 2006.07.24: Added, by Willard
 */
package mobios.type;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

/**
 * 
 * @author Willard
 */
public abstract class IndexObject implements Externalizable, Comparable<IndexObject> {
    int rowIDStart;
    int rowIDLength;

    public IndexObject() {
    }

    public IndexObject(int rowID) {
        this.rowIDStart = rowID;
        this.rowIDLength = 1;
    }

    public void setRowID(int rowID) {
        this.rowIDStart = rowID;
    }

    public int getRowID() {
        return rowIDStart;
    }

    public void setRowIDLength(int length) {
        rowIDLength = length;
    }

    /**
     * all implementing classes are required to override toString
     */
    public abstract String toString();
    
    /**
     * @return the size of the object
     */
    public abstract int size();

    /**
     * This method is necessary to implement bucketing, so that similar objects 
     * are only stored once in the database. If you do not 
     * want to implement bucketing for your object, you can just return an
     * array of length one containing this object.
     * 
     * @return an array of <code>IndexObject</code>s
     */
    public abstract IndexObject[] expand();
    
    /**
     * @param oThat
     * @return
     */
    public abstract int compareTo(IndexObject oThat);

    public void writeExternal(ObjectOutput out) throws IOException {
        out.writeInt(this.rowIDStart);
        out.writeInt(this.rowIDLength);
    }

    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        rowIDStart = in.readInt();
        rowIDLength = in.readInt();
    }

}
