/**
 * mobios.type.Symbol 2006.05.24
 * 
 * The MoBIoS Library
 * Copyright (c) 2006 The MoBIoS Group and The University of Texas at Austin.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * Change Log:
 * 2006.05.24: Modified from the original jdb package, by Willard
 */
package mobios.type;

import java.io.Externalizable;

/**
 * Interface <code>Symbol</code> is implemented for the characters of a biosequence alphabet.
 * Since biosequences often have a long life in an external representation, <code>Symbol</code>
 * alphabets should have a similarly long life, suggesting that they be immutable and easily
 * serializable. In this package, this is accomplished by implementing <code>Symbol</code> sets as
 * java enums. In addition, {@link Sequence}s using <code>Symbol</code> alphabets include a
 * <code>static public final</code> {@link Alphabet} instance, named <code>ALPHABET</code>,
 * that references the enumeration. For example, class {@link Peptide} contains the the class
 * {@link Peptide.AminoAcid} that implements <code>Symbol</code>for the common amino acids.
 * 
 * @author Jack, Rui Mao, Willard
 * @version 2003.06.04
 */
public interface Symbol extends Externalizable {
    
    /**
     * @param s
     * @return
     */
    public Symbol getSymbol(String s);

    /**
     * @return
     */
    public byte byteValue();

    /**
     * @return
     */
    public String stringValue();
}
